%% Day 1 HW solutions for Learning Catalytics questions
% Written by BB 2018/08/07
clear
close all

%% Problem 1

% How many times will the loop run?
% 

x=[];
for i=1:2:10
    x(i)=i;
end

% One way to count would be to remove the semicolon and count the number of
% times something is printed. Another would be to add a counter variable
% that increments each time. 

% The easiest, though, is to count how long the index array is, i.e. the 
% array 1:2:10

length(1:2:10)

% This returns 5


%% Problem 2

% What will x equal when the loop is done (try to do this without MATLAB)?

x=[];
for i=1:2:10
    x(i)=i;
end

%  Let's break this down step by step
%
% 1st iteration:
%   i = 1, so Matlab will run x(1) = 1;
%   Now, x is the single value 1
%
% 2nd iteration
%   i = 3, so Matlab will run x(3) = 3
%   However, there is no x(2), so Matlab will fill it with a zero.
%   Now, x = [1,0,3]
%
% 3rd iteration
%   i = 5, so Matlab will run x(5) = 5
%   Again, x is extended and filled with a zero at x(4)
%   Now, x = [1,0,3,0,5]
%
% This continues until i = 9, running x(9) = 9
% Therefore, after the loop ends x = [1,0,3,0,5,0,7,0,9]
%

%% Problem 3

% What is the chance that A will be equal to 1

A=ceil(rand*100);
if mod(A,10)~=2
    A=1;
else
    A=2;
end

% The word 'rand' is actually a function that returns a single random value 
% uniformly distributed between 0 and 1. rand*100 means the number is
% scaled to between 0 and 100. The 'ceil' function rounds numbers up to
% the nearest integer, e.g. ceil(13.093) = 14. Therefore the line 
%
%   A=ceil(rand*100)
%
% will generate a single random integer from 1 to 100. Had we used the
% 'round' function, A would range from 0 to 100, since sometimes the 
% number would be rounded down to 0. It also would not be perfectly uniform
%
% The mod function calculates the remainder after division, e.g.
%
%   mod(14,3) = 2, since 14/3 = 4 with remainder 2
%   mod(49,7) = 0, since 49/7 = 7 with no remainder
%   mod(76,10) = 6, since 76/10 = 7 with remainder 6
%
% Therefore, for any number A, mod(A,10) returns the number in the "ones"
% place. The only possible values are 0-9, 10 possible values
%
% The statement mod(A,10)~=2 means "is mod(A,10) NOT equal to 2?", i.e. it
% returns ture if mod(A,10) is not 2, and false is mod(A,10) is 2. 
%
% Therefore, A will be assigned a value of 2 if the number in the ones
% place is 2. Otherwise, it will be assigned a value of 1. Since A is
% uniformly distributed between 1 and 100, there are 10 possible numbers
% with a 2 in the ones place: 2, 12, 22, 32, 42, 52, 62, 72, 82, 92. This
% represents 10% of the possible values, so A will be set to 2 10% of the
% time. The other 90% of the time, A will be set to 1.
%
% Thus, the answer is 90% or 0.9
% 

% We could also check this numerically with a big loop
nRuns = 1e6;
nIsOne = 0 ;
for i = 1:nRuns
    if mod(ceil(rand*100),10) ~= 2
        nIsOne = nIsOne + 1;
    end
end
probIsOne = nIsOne/nRuns


%% Problem 4
% 
% Conceptually what is x going to look like? How big is the array? 
% Will there be any values equal to 1, 2, or -1? How about other numbers?
% 
x=[];
for i=1:10
    for j=1:10
        for k=1:10
            if i>j & j>k | (i==k & i==j)
                x(i,j,k) = 1;
            elseif mod(i+j+k,10)>9
                x(i,j,k) = 2;
            else
                x(i,j,k) = -1;
            end
        end
    end
end

% This is a complicated problem. The basic framework is 3 nested loops that
% fill the values in a 3D matrix. Since i, j, and k all have the values
% 1:10, then the resulting matrix will be 10x10x10
%
% The values of x depend on the if-else statements. Let's start with the 
% final statment
%
%   else
%       x(i,j,k) = -1;
%
% This statement will be executed if the two if statements above it are
% false. In other words, -1 is the "default" value that will fill the
% array unless otherwise specified.
%
% Now look at the middle statement 
%
%   mod(i+j+k,10)>9
%        x(i,j,k) = 2;
%
% In problem 3, we already said that ANY number mod 10 returns the number
% in the ones place, so a digit from 0-9. In other words, it can never be
% 10 or greater. Therefore, this statement is ALWAYS false, regardless of
% the values of i, j, and k. This means the array x will not have any
% valuues that are 2.
%
% Finally, let's unpack the first statement 
%
%   if i>j & j>k | (i==k & i==j)
%       x(i,j,k) = 1;
%
% You might remember PEMDAS as an acronym to remember order of operations. 
% Now there are two more operations
%
%   1. Inequalities: <, >, <=, >, ==, ~=, 
%   2. Compound: &, |, &&, || 
%
% Matlab ranks these operations as lower than PEMDAS operations, so one way 
% to remember this is the acronym PEMDASIC, with I for inequalities and C 
% for compounds.
%
% Now you can parse the logical statement piece by piece. 'i>j' will be
% evaluated before considering the first '&' because I comes before C. So
% the statement
%
%   i>j & j>k
%
% means "if i is greater than j AND j is greater than k". Since i=row,
% j=column, and k=sheet, then this statement refers to the indexes in the
% array where the row index is bigger than the column index, but only for 
% column indexes that are larger than the current sheet. For example, if we
% consider a small 5x5x3 array that is initially filled with 0, then the 
% 3 sheets have a value of 1 in the places:
%
%   Sheet 1              Sheet 2            Sheet 3
%       0 0 0 0 0           0 0 0 0 0           0 0 0 0 0
%       0 0 0 0 0           0 0 0 0 0           0 0 0 0 0
%       0 1 0 0 0           0 0 0 0 0           0 0 0 0 0
%       0 1 1 0 0           0 0 1 0 0           0 0 0 0 0
%       0 1 1 1 0           0 0 1 1 0           0 0 0 1 0
%
% This is when the row is bigger than the column, but only when the column
% is bigger than the sheet.
%
% The next operation is the | meaning OR, so the the whole statement will
% be evaluated as true whenever "i>j & j>k" is true OR "(i==k & i==j)" is
% true. 
%
% Now we just have to see when (i==k & i==j) is true. We have to consider
% the two statments together since they are inside a set of parentheses.
% The & operator means both statments must be true, so it means "if i is
% equal to j and if i is equal to j". Essentially it's when i=j=k, so when
% the row, column, and sheet indexes are matching. If we use the sme 5x5x3
% example from earlier, there will be a value of 1 in the places:
%
%   Sheet 1              Sheet 2            Sheet 3
%       1 0 0 0 0           0 0 0 0 0           0 0 0 0 0
%       0 0 0 0 0           0 1 0 0 0           0 0 0 0 0
%       0 0 0 0 0           0 0 0 0 0           0 0 1 0 0
%       0 0 0 0 0           0 0 0 0 0           0 0 0 0 0
%       0 0 0 0 0           0 0 0 0 0           0 0 0 0 0
%
% It places a single 1 along the main diagonal of the array, specifically
% when the row,column index matches the sheet index.
%
% Now we can put this all together. The '|' character means a 1 will be 
% placed when i>j & j>k OR when (i==k & i==j). We've already determined
% there will be no values of 2, and that everywhere else will have a -1. 
%
% Therefore, the full array will only have 1 or -1, and it will only have
% 1 in specific places. We won't show the full array, but here is the first
% sheet:
%   
%     x(:,:,1) =
% 
%          1   -1   -1   -1   -1   -1   -1   -1   -1   -1
%         -1   -1   -1   -1   -1   -1   -1   -1   -1   -1
%         -1    1   -1   -1   -1   -1   -1   -1   -1   -1
%         -1    1    1   -1   -1   -1   -1   -1   -1   -1
%         -1    1    1    1   -1   -1   -1   -1   -1   -1
%         -1    1    1    1    1   -1   -1   -1   -1   -1
%         -1    1    1    1    1    1   -1   -1   -1   -1
%         -1    1    1    1    1    1    1   -1   -1   -1
%         -1    1    1    1    1    1    1    1   -1   -1
%         -1    1    1    1    1    1    1    1    1   -1
%
% Notice that x(1,1,1) = 1, along with the lower-triangular part of this
% matrix (minus the first column). Everywhere else is -1.

%% Problem 5

% Look up the help on the while loop. What does this program do? Think big 
% picture.

a=round(rand*100)
p=0;
while (p==0)
    p=1;
    for x=2:a/2
        if mod(a,x)==0
            p=0;
        end
    end
    a=a+1;
end
a-1


% The first line a=round(rand*100) will generate a random integer from 0 to
% 100. The variable p is then set to 0
%
% The while loop has a conditional statement like an if-else block, but it
% will repeat the code inside it as long as the condition is true. Since we
% set p to 0, this is initially true and the code inside will run
%
% Inside the loop, p is set to 1, and then we begin a for loop from 2 to
% a/2. In each iteration of the for loop, we can reset p to 0, but only if
% a/x has no remainder, i.e. x evenly divides a. This will happen for most
% numbers
%
% After the for loop ends, a is incremented by 1 and the while loop checks
% again to see if p is still 0. If it isn't, the loop ends and the value
% a-1 is printed out
%
% Essentially, this loop will keep incrementing a by 1 and repeating until
% a specific condition is met: that NO number from 2 to a/2 evenly divides
% a. In other words, when 'a' cannot be evenly divided by another integer. 
%
% This is the definition of a prime number
%
% Therefore, the loop will increment 'a' until it is prime. The final a-1
% line will account for the last iteration of the loop when 'a' was prime 
% but a=a+1 added an extra 1. Therefore, the line a-1 will return the
% prime number.
%
% Put simply, this code will generate a random number 'a' from 0-100 and 
% then overwrite 'a' with the smallest prime number greater than or equal 
% to the intial random value.
%
% Interestingly, this code actually has a bug. If 'a' is initially 0 or 1
% (which are both possible because we used 'round' instead of 'ceil'), then 
% the for loop will never run and the code will return the original value 0
% or 1. Technically, neither of these are prime numbers. This code would
% need some additional error checking to be strictly a prime number
% generator
%% Problem 6

% Assume you have a 10 x 10 array named a 
%
% Using a single line of code: to each element of the sixth and eighth row
% of the array a, add double the value of the element in the sixth row, 
% sixth column of array a.


% I'll create a 10x10 array just so the code below works.
a = randi(5,10);

% There are several ways you can answer this. Technically, you could even
% use loops and put everything on a single illegible line
%
% Here's what we were looking for:

a([6 8],:) = a([6 8],:) + 2*a(6,6) 

% This line uses [6 8] as the row argument while indexing to get the 6th
% and 8th rows of a. The ':' as the column argument will index all of the
% columns. So a(6,:) is the entire 6th row, and a([6 8],:) is the entire
% 6th and 8th rows
%
% The question says to add twice the value of a(6,6), which we can get with 
% simply 2*a(6,6). Matlab's order of operations will ensure the
% multiplication is done first.


%% Problem 7

%Assuming you have an array named a, save a new array b1 as the entire 
% array a to the power of the value in the fourth row, fifth column of 
% array a.  (x^y is x to the power y) . 

b1 = a^a(4,5)

% This question is trying get the difference between a^x and a.^x. The
% first uses matrix multiplication to multiply a by itself x times. The
% second will raise each element in a to the xth power
%
% Since this question says "the entire array a to the power of ...", the
% correct answer uses a^x


%% Problem 8

% Assuming you have an array named a, save a new array b2 as each element 
% of the array a to the power of the value in the fourth row, fifth column 
% of array a.

b2 = a.^a(4,5)

% This problem asks you to exponentiate each element of a, so the notation
% uses the dot, e.g. a.^x

%% Problem 9

% What is the average value of every third column (1st , 4th , 7th , etc.)
% of an array named a?

mean(a(:,1:3:end)) 

% The argument to the mean function here a(:,1:3:end) uses the colon
% operator to return every 3rd column. Note that we didn't specify how big
% a was, so you need the 'end' keyword to make sure you get every column
%
% When a 2D array is passed to the mean function, it takes the average of
% each column by default. So we don't need to specify the dimension.
% However, the line

mean(a(:,1:3:end),1) 

% that specifies the dimension is also correct


%% Problem 10

% What does the ' command do when placed after an array 
% (try the command out on several different arrays to learn)?

% The ' command takes the "transpose" of an array, meaning it switches 
% each row to a column to a row and each row to a column. In other words
% 
%   a(i,j)' = a(j,i)
%
% For any index i,j.
%
% Here's a question for you: what does a' return if a is a 3D array?
%


%% Problem 11 - 15

% For problems 11 - 15, see the 'Yeast Showdown_I_solution.m' file. It
% contains a detailed walkthrough of Yeast Showdown I. Make sure you run
% the solution file from the same folder as the images.